#!/usr/bin/env python
# coding: utf-8
# author: ei-grad

import Image, ImageDraw
from pycopia.OS.Linux import Input, event

# таблица преобразования keycodes в ascii и названия клавиш
KEYS = dict((j, i) for i, j in event._VALUEMAP.items())


def key(code):
    """
    Для клавиш, названия которых записываются одним символом (буквы, цифры,
    разные знаки) просто выводим этот символ. Для служебных клавиш выводим их
    названия заключенными в угловые скобки.
    """
    return KEYS[code] if len(KEYS[code]) == 1 else '<%s>' % KEYS[code]

def print_keydowns(events):
    """
    Выводим нажатия клавиш.

    Значение e.value == 1 соответствует событию key_down.
    """
    print(''.join([key(e.code) for e in events
                   if e.evtype == event.EV_KEY and e.value == 1]))


def draw_path(events, out=None, clicks_only=False):
    """
    Рисует путь пройденный мышкой, обозначая цифрами места нажатия кнопок.

    @param out: имя файла, в который нужно записать картинку с результатом
    @param clicks_only: не рисовать путь, только цифры
    """

    # создаем изображение и рисовалку для него
    img = Image.new('RGB', (4000, 4000))
    draw = ImageDraw.Draw(img)

    # заполняем изображение белым цветом
    draw.rectangle((0, 0, 4000, 4000), fill='white')

    # начальная позиция мышки
    x = y = 2000

    # номер нажатия
    n = 0

    # флажок для определения key_down/key_up
    q = True

    for t, c, v in [(e.evtype, e.code, e.value) for e in events]:

        # относительное изменение позиции мышки
        if t == event.EV_REL:
            new_x, new_y = x, y
            if c == 0:
                new_x += v
            else:
                new_y += v
            if draw_path:
                draw.line((x, y, new_x, new_y), fill='black')
            x, y = new_x, new_y

        # нажатие или отпускание клавиши мышки
        if t == 4:
            # с мышкой непонятно какое из событий есть нажатие, а какое -
            # отпускание, так что рисуем только нечетные по порядку события
            if n % 2:
                draw.text((x, y), str(n/2), fill='red')
            n += 1
            q = not q

    if out is not None:
        img.save(out)

    img.show()

if __name__ == "__main__":

    # загружаем события из файлов
    kbd_events = list(Input.EventFile('keyboard.data').readall())
    mouse_events = list(Input.EventFile('mouse.data').readall())

    # выводим нажатия клавиш
    print_keydowns(kbd_events)

    # опытным путем мы определили диапазон событий мышки, связанных со вводом
    # пароля на http://keyboard.yandex.ru
    passwd_events = mouse_events[1758:4184]

    # рисуем путь, пройденный мышкой, сохраняем в result.jpg
    draw_path(passwd_events, "result.jpg")
